/******************************************************************************
** Compagny     : 4JS
** Program      : fglreportc.c
** Date         : 10/01/1996
** Version      : 1.00
**
** Titel        : Managment of report:  Previous/Next Page data stores
******************************************************************************/

/******************************************************************************
**			     INCLUSIONS + MACROS
******************************************************************************/

#include<stdio.h>
#include<f2c/r_c.h>
#ifndef NULL
#define NULL 0
#endif

/******************************************************************************
**			       GLOBALS VARIABLES
******************************************************************************/

static long nb_page=0;

struct page {
	char **lines;
	long length;
	struct page *next;
}; 

static struct page *curr_report;
static struct page *lastpage;

/******************************************************************************
**			     	FUNCTIONS
******************************************************************************/

/*******************************************************************************
** FUNCTION Cfgl_report_initpage : Initialize a Page structure
**                                 
** Parameters :
**  curr_nb : Current number of page
** Returnings :
**  page    : Structure page
*******************************************************************************/

static struct page *Cfgl_report_initpage(curr_nb)

long curr_nb;

{
 struct page *page;

 if ( curr_nb > nb_page + 1) {
   fprintf(stderr,"Bad paramaters in fgl_report functions\n");
   exit(1);
 }

 page = (struct page *)malloc(sizeof(struct page));
 if ( nb_page == 0 ) {
   curr_report = page;
   lastpage    = page;
 } else {
   lastpage->next = page;
   lastpage       = page;
 }

 page->length = 0;
 page->next   = NULL;

 nb_page++;

 return(page);

} /* END FUNCTION Cfgl_report_initpage */

/*******************************************************************************
** FUNCTION Cfgl_report_currpage : Give a page
** Parameters :
**  curr_nb : Number of parameters
** Returnings :
**  page    : Current page
*******************************************************************************/

static struct page *Cfgl_report_currpage(curr_nb)

long curr_nb;

{
 struct page *page;
 register long i;

 if ( curr_nb > nb_page )
   page = Cfgl_report_initpage(curr_nb);

 for ( i = 0 , page = curr_report; i < curr_nb-1;i++) {
   page = page->next;
 }

 return(page);

} /* END FUNCTION Cfgl_report_currpage */


/*******************************************************************************
** FUNCTION Cfgl_report_storelength : Store length of currpage
** Parameters :
**  curr_page : Current page
**  length    : Length of page
** Returnings :
**  none
*******************************************************************************/

int Cfgl_report_storelength(curr_page,length)

long curr_page;
long length;

{
 struct page *page;

 page = Cfgl_report_currpage(curr_page);

 page->length = length;
 page->lines  = (char **)malloc(sizeof(char *)*length);
 return(0);

} /* END FUNCTION Cfgl_report_storelength */

/*******************************************************************************
** FUNCTION Cfgl_report_storeline : Store a line of currpage
** Parameters :
**  curr_page : Current page
**  ind       : Index in Page
**  line      : Current line
** Returnings :
**  nbret : Number of returning: 0
*******************************************************************************/

int Cfgl_report_storeline(curr_page,ind,line)

long curr_page;
long ind;
char *line;

{
 struct page *page;

 page = Cfgl_report_currpage(curr_page);

 if ( ind > page->length ) {
   fprintf(stderr,"fgl_report_storeline: index greater than length of page\n");
   exit(1);
 } else if ( ind < 1 ) {
   fprintf(stderr,"fgl_report_storeline: index less than 1\n");
   exit(1);
 }

 page->lines[ind-1]   = (char *)strdup(line);
 return(0);

} /* END FUNCTION Cfgl_report_storeline */

/*******************************************************************************
** FUNCTION Cfgl_report_maxpage : Give index of greater stored page
** Parameters :
**  none
** Returnings :
**  maxpage: index of greater stored page
*******************************************************************************/

long Cfgl_report_maxpage()

{

 return(nb_page);

} /* END FUNCTION Cfgl_report_storeline */

/*******************************************************************************
** FUNCTION Cfgl_report_givelength : Give length of currpage
** Parameters :
**  curr_page : Current page
** Returnings :
**  length:     Length of page
*******************************************************************************/

long Cfgl_report_givelength(curr_page)

long curr_page;

{
 struct page *page;

 page = Cfgl_report_currpage(curr_page);

 return (page->length );

} /* END FUNCTION Cfgl_report_givelength */

/*******************************************************************************
** FUNCTION Cfgl_report_giveline : Give line of currpage
** Parameters :
**  curr_page : Current page
**  ind       : Index of page
** Returnings :
**  line:     Line of page
*******************************************************************************/

char *Cfgl_report_giveline(curr_page,ind)

long curr_page;
long ind;

{
 struct page *page;

 page = Cfgl_report_currpage(curr_page);

 if ( ind > page->length || ind < 0 ) {
   fprintf(stderr,"fgl_report_giveline: Bad index in page\n");
   exit(1);
 }

 return (page->lines[ind-1] );

} /* END FUNCTION Cfgl_report_giveline */

/*******************************************************************************
** FUNCTION Cfgl_report_freedata : Free data of current report
** Parameters :
**  none
** Returnings :
**  none
*******************************************************************************/

int Cfgl_report_freedata()

{
 register struct page *page;
 register long i;


 for (page=curr_report; page !=NULL; ) {
   for (i=0; i < page->length; 	i++) {
     free(page->lines[i]);
   }
   free(page->lines);
   lastpage=page;
   page = page->next;
   free(lastpage);
 }

 nb_page = 0;
 return ( 0 );

} /* END FUNCTION Cfgl_report_freedata */

/******************************************************************************
**			     	CALLING-4GL-FUNCTIONS
******************************************************************************/

/*******************************************************************************
** FUNCTION fgl_report_storelength : Store length of currpage
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: curr_page : Current page
**                  length    : Length of page
** Returnings :
**  nbret : Number of returning: 0
*******************************************************************************/

int fgl_report_storelength(nargs)

int nargs;

{
 long curr_page;
 long length;

 if (nargs != 2 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_storelength\n");
   exit(1);
 }
 poplong(&length);
 poplong(&curr_page);
 Cfgl_report_storelength(curr_page,length);
 return(0);

} /* END FUNCTION fgl_report_storelength */

/*******************************************************************************
** FUNCTION fgl_report_storeline : Store length of currpage
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: curr_page : Current page
**                  ind       : Current index
**                  line      : Current line
** Returnings :
**  nbret : Number of returning: 0
*******************************************************************************/

int fgl_report_storeline(nargs)

int nargs;

{
 long curr_page;
 long ind;
 char line[513];

 if (nargs != 3 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_storeline\n");
   exit(1);
 }

 popquote(line,513);
 poplong(&ind);
 poplong(&curr_page);

 Cfgl_report_storeline(curr_page,ind,line);
 return(0);

} /* END FUNCTION fgl_report_storeline */

/*******************************************************************************
** FUNCTION fgl_report_maxpage : Give index of greater stored page
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: none
** Returnings :
**  nbret : Number of returning: 1
*******************************************************************************/

int fgl_report_maxpage(nargs)

int nargs;

{

 if (nargs != 0 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_maxpage\n");
   exit(1);
 }

 retlong(Cfgl_report_maxpage());
 return(1);

} /* END FUNCTION fgl_report_maxpage */

/*******************************************************************************
** FUNCTION fgl_report_minpage : Give index of minimum stored page
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: none
** Returnings :
**  nbret : Number of returning: 1
*******************************************************************************/

int fgl_report_minpage(nargs)

int nargs;

{

 if (nargs != 0 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_minpage\n");
   exit(1);
 }

 retlong(1);
 return(1);

} /* END FUNCTION fgl_report_minpage */

/*******************************************************************************
** FUNCTION fgl_report_givelength : Give length of currpage
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: curr_page : Current page
** Returnings :
**  nbret : Number of returning: 1
*******************************************************************************/

int fgl_report_givelength(nargs)

int nargs;

{
 long curr_page;

 if (nargs != 1 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_givelength\n");
   exit(1);
 }
 poplong(&curr_page);
 retlong(Cfgl_report_givelength(curr_page));
 return(1);

} /* END FUNCTION fgl_report_givelength */

/*******************************************************************************
** FUNCTION fgl_report_giveline : Give line of currpage
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: curr_page : Current page
**                  index     : Index in page
** Returnings :
**  nbret : Number of returning: 1
*******************************************************************************/

int fgl_report_giveline(nargs)

int nargs;

{
 long curr_page;
 long ind;

 if (nargs != 2 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_giveline\n");
   exit(1);
 }
 poplong(&ind);
 poplong(&curr_page);
 retquote(Cfgl_report_giveline(curr_page,ind));
 return(1);

} /* END FUNCTION fgl_report_giveline */

/*******************************************************************************
** FUNCTION fgl_report_freedata : Free data of current report
** Parameters :
**  nargs : Number of parameters.
**          In 4GL: none
** Returnings :
**  nbret : Number of returning: 1
*******************************************************************************/

int fgl_report_freedata(nargs)

int nargs;

{
 long curr_page;
 long ind;

 if (nargs != 0 ) {
   fprintf(stderr,"Bad number of parameters in function fgl_report_freedata\n");
   exit(1);
 }
 Cfgl_report_freedata();
 return(0);

} /* END FUNCTION fgl_report_freedata */

CFACE(fgl_report_storelength,2,0);
CFACE(fgl_report_storeline,3,0);
CFACE(fgl_report_maxpage,0,1);
CFACE(fgl_report_minpage,0,1);
CFACE(fgl_report_givelength,1,1);
CFACE(fgl_report_giveline,2,1);
CFACE(fgl_report_freedata,0,0);
